/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "fedmap.h"

#include <sstream>

#include "io.h"

#include "cmdform.h"
#include "infra.h"
#include "location.h"
#include "main.h"
#include "map_parser.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------


const int FedMap::MAP_SIZE = 64;

FedMap::FedMap()
{
	LoadMapGfx();
	infra = new Infra;
	current_loc = -1;
}

FedMap::~FedMap()
{
	Write();
	for(LocIndex::iterator iter = loc_index.begin();iter != loc_index.end();iter++)
		delete iter->second;
	delete map_gfx;
}


void	FedMap::AddFactory(const AttribList& attrib)
{
	infra->AddFactory(attrib);
}

void	FedMap::AddLoc(Location *loc,bool update)
{
	current_loc = loc->LocNum();
	LocIndex::iterator	iter = loc_index.find(current_loc);
	if(iter != loc_index.end())
	{
		Location *temp = iter->second;
		loc_index.erase(iter);
		delete temp;
	}

	loc_index[current_loc] = loc;

	if(update)
	{
		CentreMap(current_loc,MainForm->DetailGrid);
		MainForm->DetailGrid->Refresh();
		CentreMap(current_loc,MainForm->MapGrid);
		MainForm->MapGrid->Refresh();
	}
}

void	FedMap::CentreMap(int loc_no,TDrawGrid *grid)
{
	if(loc_no == -1)
		loc_no = current_loc;
	int	col = (loc_no % MAP_SIZE) * 2;
	int	row = (loc_no/MAP_SIZE) * 2;
	grid->Col = col + 1;
	grid->Row = row + 1;
	int left = col - grid->VisibleColCount/2 + 1;
	int top = row - grid->VisibleRowCount/2 + 1;
	if(left < 0)
		left = 0;
	if(top < 0)
		top = 0;
	if(left > (grid->ColCount - grid->VisibleColCount))
		left = grid->ColCount - grid->VisibleColCount;
	if(left > (grid->RowCount - grid->VisibleRowCount))
		left = grid->RowCount - grid->VisibleRowCount;

	grid->LeftCol = left;
	grid->TopRow = top;

}

int	FedMap::CheckDiagonalLink(int col,int row)
{
	int	diag = 0;

	// possible sw/ne link
	if(CheckExit(col - 1,row + 1,Location::NE))
		diag = 5 * Location::TILE_SIZE;
	if(CheckExit(col + 1,row - 1,Location::SW))
		diag = 5 * Location::TILE_SIZE;

	// possible se/nw link
	if(CheckExit(col - 1,row - 1,Location::SE) ||
							CheckExit(col + 1,row + 1,Location::NW))
	{
		if(diag == 0)
			diag = 6* Location::TILE_SIZE;
		else
			diag = 7 * Location::TILE_SIZE;	// crossed diagonals!
	}
	return(diag);
}

bool	FedMap::CheckExit(int col,int row,int dir)
{
	int	loc_no = (row/2) * MAP_SIZE + col/2;
	LocIndex::iterator	iter = loc_index.find(loc_no);
	if(iter == loc_index.end())				// not a real location
		return(false);
	else
		return(iter->second->IsAnExit(dir));
}

int	FedMap::CheckLeftRightLink(int col,int row)
{
	if(CheckExit(col - 1,row,Location::EAST))
		return(4 * Location::TILE_SIZE);
	if(CheckExit(col + 1,row,Location::WEST))
		return(4 * Location::TILE_SIZE);
	return(0);
}

int	FedMap::CheckUpDownLink(int col,int row)
{
	if(CheckExit(col,row - 1,Location::SOUTH))
		return(3 * Location::TILE_SIZE);
	if(CheckExit(col,row + 1,Location::NORTH))
		return(3 * Location::TILE_SIZE);
	return(0);
}

void	FedMap::Clear()
{
	if((loc_index.size() > 0) && (infra->Name() != ""))
		Write();
	for(LocIndex::iterator iter = loc_index.begin();iter != loc_index.end();iter++)
		delete iter->second;
	loc_index.clear();

	MainForm->DetailGrid->Refresh();
	MainForm->MapGrid->Refresh();
	MainForm->MapMemo->Clear();
}

void	FedMap::DisplayInfra()
{
	infra->Display();
}

void	FedMap::DrawCell(int col,int row,TRect &rect,TDrawGrid *grid)
{
	TRect	source;
	int	loc_num;

	source.Left = 0;
	if(((col % 2) == 1) && ((row % 2) == 1))				// a potential location
	{
		loc_num = (row/2) * MAP_SIZE + col/2;
		if(loc_index.find(loc_num) != loc_index.end())	// location does exist...
		{
			if(loc_num == current_loc)
				source.Left = 1 * Location::TILE_SIZE;
			else
				source.Left = 2 * Location::TILE_SIZE;
		}
	}
	else																// a potential link between locs
	{
		if((col % 2) == 1)
			source.Left = CheckUpDownLink(col,row);
		else
		{
			if((row % 2) == 1)
				source.Left = CheckLeftRightLink(col,row);
			else
			{
				if(((row % 2) == 0) && ((col % 2) == 0))
					source.Left = CheckDiagonalLink(col,row);
			}
		}
	}

	// now draw the cell
	source.Right = source.Left + Location::TILE_SIZE;
	source.Top = 0;
	source.Bottom = source.Top + Location::TILE_SIZE;
	grid->Canvas->CopyRect(rect,map_gfx->Canvas,source);
}

bool	FedMap::Load()
{
	std::string full_file_name = MakeMapFileName();
	if(access(full_file_name.c_str(),0) != 0)
		return(false);
	MapParser *parser = new MapParser(full_file_name);
	parser->Parse();
	delete parser;
	return(true);
}

void	FedMap::LoadMapGfx()
{
	map_gfx = new Graphics::TBitmap;
	std::string	file_name = MainForm->HomeDir() + "\\links.bmp";
	map_gfx->LoadFromFile(file_name.c_str());
}

void	FedMap::MovementKeys(WORD &key)
{
	switch(key)
	{
		case VK_MULTIPLY:	CommandForm->Visible = true;									break;
		case VK_NUMPAD9:
		case VK_PRIOR:		MainForm->OutputText("ne"); MainForm->Send("ne\n");	break;
		case VK_NUMPAD3:
		case VK_NEXT:		MainForm->OutputText("se"); MainForm->Send("se\n");	break;
		case VK_NUMPAD1:
		case VK_END:		MainForm->OutputText("sw"); MainForm->Send("sw\n");	break;
		case VK_NUMPAD7:
		case VK_HOME:		MainForm->OutputText("nw"); MainForm->Send("nw\n");	break;
		case VK_NUMPAD4:
		case VK_LEFT:		MainForm->OutputText("w");  MainForm->Send("w\n");		break;
		case VK_NUMPAD8:
		case VK_UP:			MainForm->OutputText("n");  MainForm->Send("n\n");		break;
		case VK_NUMPAD6:
		case VK_RIGHT:		MainForm->OutputText("e");  MainForm->Send("e\n");		break;
		case VK_NUMPAD2:
		case VK_DOWN:		MainForm->OutputText("s");  MainForm->Send("s\n");		break;
		case VK_NUMPAD0:
		case VK_INSERT:	MainForm->OutputText("d"); MainForm->Send("d\n");		break;
		case VK_DECIMAL:
		case VK_DELETE:	MainForm->OutputText("u"); MainForm->Send("u\n");		break;
		case VK_NUMPAD5:
		case VK_ADD:		MainForm->OutputText("board"); MainForm->Send("board\n");	break;
	}
}

const std::string&	FedMap::MakeMapFileName()
{
	static std::string	full_file_name;

	std::string file_name = "\\maps\\" + infra->Name() + ".xml";
	int len = file_name.length();
	for(int count  = 0;count < len;count++)
	{
		if(file_name[count] == ' ')
			file_name[count] = '_';
		file_name[count] = std::tolower(file_name[count]);
	}
	full_file_name = MainForm->HomeDir() + file_name;
	return(full_file_name);
}

void	FedMap::MapInfo(const AttribList& attrib)
{
	infra->AddEconomy(attrib);
}

void	FedMap::Name2Status(int col,int row)
{
	if(((col % 2) == 1) && ((row % 2) == 1))				// a potential location
	{
		LocIndex::iterator iter = loc_index.find((row/2) * MAP_SIZE + col/2);
		if(iter != loc_index.end())							// real location
			MainForm->StatusBar->Panels->Items[3]->Text = (*iter).second->Name().c_str();
		else
			MainForm->StatusBar->Panels->Items[3]->Text = "";

	}
	else
		MainForm->StatusBar->Panels->Items[3]->Text = "";
}

void	FedMap::NewMap(const AttribList& attrib)
{
	MainForm->MapImage->Visible = false;
	MainForm->DetailGrid->Visible = true;
	Clear();
	infra->NewInfra(attrib);
	MainForm->LocListBox->Clear();
	Load();
	MainForm->DetailGrid->Refresh();
	MainForm->MapGrid->Refresh();
	current_loc = -1;
}

void	FedMap::RemoveFactory(const AttribList& attrib)
{
	infra->RemoveFactory(attrib);
}

void	FedMap::UpdateWorkers(const AttribList& attrib)
{
	infra->UpdateWorkers(attrib);
}

void	FedMap::Write()
{
	if((infra->Name() == "") || (loc_index.size() < 2))
		return;

	std::string full_file_name = MakeMapFileName();
	std::ofstream	file(full_file_name.c_str(),std::ios::out | std::ios::trunc);
	if(file)
	{
		file << "<?xml version=\"1.0\"?>\n\n";
		infra->Write(file);
		for(LocIndex::iterator iter = loc_index.begin();iter != loc_index.end();iter++)
			iter->second->Write(file);
		file << "\n</map>" << std::endl;
	}
	else
	{
		MainForm->OutputDebug("Can't open file for writing:");
		MainForm->OutputDebug(full_file_name);
	}
}




